@@ -1,7 +1,7 @@
 /*
  * mdadm - manage Linux "md" devices aka RAID arrays.
  *
- * Copyright (C) 2001-2002 Neil Brown <neilb@cse.unsw.edu.au>
+ * Copyright (C) 2001-2006 Neil Brown <neilb@suse.de>
  *
  *
  *    This program is free software; you can redistribute it and/or modify
@@ -34,7 +34,7 @@
 #include	"md_u.h"
 #include	"md_p.h"
 
-int Kill(char *dev, int force)
+int Kill(char *dev, int force, int quiet)
 {
 	/*
 	 * Nothing fancy about Kill.  It just zeroes out a superblock
@@ -47,13 +47,17 @@ int Kill(char *dev, int force)
 		
 	fd = open(dev, O_RDWR|O_EXCL);
 	if (fd < 0) {
-		fprintf(stderr, Name ": Couldn't open %s for write - not zeroing\n",
-			dev);
+		if (!quiet)
+			fprintf(stderr, Name ": Couldn't open %s for write - not zeroing\n",
+				dev);
+		close(fd);
 		return 1;
 	}
 	st = guess_super(fd);
 	if (st == NULL) {
-		fprintf(stderr, Name ": Unrecognised md component device - %s\n", dev);
+		if (!quiet)
+			fprintf(stderr, Name ": Unrecognised md component device - %s\n", dev);
+		close(fd);
 		return 1;
 	}
 	rv = st->ss->load_super(st, fd, &super, dev);
@@ -63,13 +67,15 @@ int Kill(char *dev, int force)
 		mdu_array_info_t info;
 		info.major_version = -1; /* zero superblock */
 		free(super);
-		st->ss->init_super(st, &super, &info, 0, "");
+		st->ss->init_super(st, &super, &info, 0, "", NULL, NULL);
 		if (st->ss->store_super(st, fd, super)) {
-			fprintf(stderr, Name ": Could not zero superblock on %s\n",
-				dev);
+			if (!quiet)
+				fprintf(stderr, Name ": Could not zero superblock on %s\n",
+					dev);
 			rv = 1;
 		} else if (rv) {
-			fprintf(stderr, Name ": superblock zeroed anyway\n");
+			if (!quiet)
+				fprintf(stderr, Name ": superblock zeroed anyway\n");
 			rv = 0;
 		}
 	}
